@@ -1,6 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-# $Id: StatisticsHandler.py 10079 2017-07-03 13:52:00Z David $
+# $Id: StatisticsHandler.py 10083 2017-07-04 08:36:29Z David $
 #
 # Copyright (c) 2012 Nuwa Information Co., Ltd, All Rights Reserved.
 #
@@ -12,8 +12,8 @@
 # limitations under the License.
 #
 # $Author: David $
-# $Date: 2017-07-03 21:52:00 +0800 (Mon, 03 Jul 2017) $
-# $Revision: 10079 $
+# $Date: 2017-07-04 16:36:29 +0800 (二, 04  7 2017) $
+# $Revision: 10083 $
 
 from datetime import timedelta, datetime
 
@@ -25,6 +25,7 @@ from Iuppiter.Encoding import _unicode
 
 from Theophrastus.models import Newsletter
 from Theophrastus.models import ContactMailingStatus
+from collections import OrderedDict
 
 _STATUS = [
     'deliverie', 'error', 'invaild', 'open', 'openOnSite', 'click',
@@ -35,49 +36,36 @@ STATUS_DICT = dict(ContactMailingStatus.STATUS_CHOICES)
 STATUS_NAMES = STATUS_DICT.values()
 STATUS_CODE = STATUS_DICT.keys()
 
-def getDateRange(rangeName='week', defaultStart=None):
+def getDateRange(startDate, endDate):
     """
     Get range of dates.
-
-    week: from last week to today.
-    month: from last months to today.
-    3month: from last three months to today.
-
-    @param rangeName The range of name.
     """
-    if not rangeName.strip():
-        rangeName = 'week'
-
-    endDate = datetime.now()
-
-    rangeMapping = {
-        'week': 7,
-        'month': int(1 * (365 / 12)),
-        '3month': int(3 * (365 / 12)),
-    }
-
-    if rangeName in rangeMapping:
-        deltaNumber = rangeMapping[rangeName]
-        days = [endDate.strftime('%Y-%m-%d')]
-        for i in range(1, deltaNumber):
-            d = endDate - timedelta(days=i)
-            if defaultStart and defaultStart > d:
-                break
-
+    if isinstance(startDate, str):
+        if startDate != "NaN":
+            startDate = datetime.strptime(startDate, '%Y/%m/%d')
+    if isinstance(endDate, str):
+        if endDate != "NaN":
+            endDate = datetime.strptime(endDate, '%Y/%m/%d')
+    
+    deltaDays = endDate - startDate
+    daysAmount = deltaDays.days
+    
+    def _createDate(daysAmount):
+        days = [startDate.strftime('%Y-%m-%d')]
+        for i in range(1, daysAmount):
+            d = startDate + timedelta(days=i)
             d = d.strftime('%Y-%m-%d')
             days.append(d)
 
-        days.reverse()
-
         class DateRange(object):
             def __init__(self, days):
                 self._days = days
                 self.startDate = '%s 00:00:00' % days[0]
                 self.endDate = '%s 23:59:59' % days[-1]
-
+                
         return DateRange(days)
-    else:
-        return None
+
+    return _createDate(daysAmount)
 
 #def countStatus(queryset):
 #    """
@@ -267,6 +255,9 @@ def getEmptyDataStructure():
         'reject': 0,
     }
 
+def getEmptyDataList():
+    return []
+
 def countStatus(queryset):
     """
     Calculate the status number of occurrences.
@@ -296,6 +287,10 @@ def countStatus(queryset):
     clickLogs = set()
     unsubscriptionNumber = set()
 
+    openId = []
+    openOnSiteId = []
+    clickId = []
+
     bouncedLogs = set()
     complainLogs = set()
     rejectedLogs = set()
@@ -310,7 +305,22 @@ def countStatus(queryset):
         elif status == ContactMailingStatus.INVALID:
             invaildLogs.add(id)
         elif status == ContactMailingStatus.UNSUBSCRIPTION:
-            unsubscriptionNumber.add(id)
+            unsubscriptionNumber.add(id)        
+        elif status == ContactMailingStatus.OPENED:
+            openLogs.add(id)
+            openId.append(id)     
+        elif status == ContactMailingStatus.OPENED_ON_SITE:
+            openOnSiteLogs.add(id)
+            openOnSiteId.append(id)
+            if not id in openLogs:
+                openLogs.add(id)
+                openId.append(id)              
+        elif status == ContactMailingStatus.LINK_OPENED:
+            clickLogs.add(id)
+            clickId.append(id)
+            if not id in openLogs:
+                openLogs.add(id)
+                openId.append(id)  
         ##
         elif status == ContactMailingStatus.BOUNCED:
             bouncedLogs.add(id)
@@ -321,44 +331,80 @@ def countStatus(queryset):
     
     # formula reference: emencia.django.utils.statistics.py
     # openings
-    openings = queryset.filter(Q(status=ContactMailingStatus.OPENED) | 
-                               Q(status=ContactMailingStatus.OPENED_ON_SITE))
-    openingsByLinks = len(
-        set(queryset.filter(status=ContactMailingStatus.LINK_OPENED).exclude(
-        contact__in=openings.values_list('contact', flat=True)
-    ).values_list('contact', flat=True)))
-    uniqueOpenings = len(set(openings.values_list('contact', flat=True))) + \
-                     openingsByLinks
-                          
-    # on site openings
-    onSiteOpenings = queryset.filter(
-       status=ContactMailingStatus.OPENED_ON_SITE)
-    uniqueOnSiteOpenings = len(set(onSiteOpenings.values_list(
-                              'contact', flat=True)))
-            
-    # clicked
-    clickedLinks = queryset.filter(status=ContactMailingStatus.LINK_OPENED)
-    uniqueClicked = len(set(clickedLinks.values_list(
-                              'contact', flat=True)))
-
-    return {
+#     openings = queryset.filter(Q(status=ContactMailingStatus.OPENED) | 
+#                                Q(status=ContactMailingStatus.OPENED_ON_SITE))
+#     openingsByLinks = len(
+#         set(queryset.filter(status=ContactMailingStatus.LINK_OPENED).exclude(
+#         contact__in=openings.values_list('contact', flat=True)
+#     ).values_list('contact', flat=True)))
+#     uniqueOpenings = len(set(openings.values_list('contact', flat=True))) + \
+#                      openingsByLinks
+#                           
+#     # on site openings
+#     onSiteOpenings = queryset.filter(
+#        status=ContactMailingStatus.OPENED_ON_SITE)
+#     uniqueOnSiteOpenings = len(set(onSiteOpenings.values_list(
+#                               'contact', flat=True)))
+#             
+#     # clicked
+#     clickedLinks = queryset.filter(status=ContactMailingStatus.LINK_OPENED)
+#     uniqueClicked = len(set(clickedLinks.values_list(
+#                               'contact', flat=True)))
+    return [{
         'sentTest': len(sentTestNumber),
         'deliverie': len(sentNumber),
         'error': len(errorLogs),
         'invaild': len(invaildLogs),
-        'open': uniqueOpenings,
-        'openOnSite': uniqueOnSiteOpenings,
-        'click': uniqueClicked,
+        'open': len(openLogs),
+        'openOnSite': len(openOnSiteLogs),
+        'click': len(clickLogs),
         'unsubscribe': len(unsubscriptionNumber),
         'bounce': len(bouncedLogs),
         'complain': len(complainLogs),
-        'reject': len(rejectedLogs),
-    }
+        'reject': len(rejectedLogs),        
+        
+        },
+        {
+         'openId': openId,
+         'openOnSiteId': openOnSiteId,
+         'clickId': clickId
+        }            
+    ]
 
 from django.db.models import Q
 
 from emencia.django.newsletter.models import ContactMailingStatus as Status
 
+def getStartDateEndDate():
+    startDate = ContactMailingStatus.objects.order_by('creation_date')[0
+                ].creation_date.strftime('%Y-%m-%d')
+    endDate = ContactMailingStatus.objects.order_by('-creation_date')[0
+               ].creation_date.strftime('%Y-%m-%d')
+    return startDate, endDate
+
+def getAllDataDate():    
+    firstDate = ContactMailingStatus.objects.order_by('creation_date')[0
+                ].creation_date
+    lastDate = ContactMailingStatus.objects.order_by('-creation_date')[0
+               ].creation_date
+    
+    deltaDays= lastDate - firstDate
+    daysAmount = deltaDays.days       
+    
+    days = [firstDate.strftime('%Y-%m-%d')]
+    for i in range(1, daysAmount):
+        d = firstDate + timedelta(days=i)
+        d = d.strftime('%Y-%m-%d')
+        days.append(d)
+
+    class DateRange(object):
+        def __init__(self, days):
+            self._days = days
+            self.startDate = '%s 00:00:00' % days[0]
+            self.endDate = '%s 23:59:59' % days[-1]
+            
+    return DateRange(days)
+
 def getStatisticsTotalData(reqestGetData, subscriberCount, testSubscriberCount, 
                            conditions):
     """
@@ -368,15 +414,20 @@ def getStatisticsTotalData(reqestGetData, subscriberCount, testSubscriberCount,
     @param slug Newsletter slug.
     @param request Request instance.
     """
-    period = reqestGetData.get('period', 'week')
+    
     dataType = reqestGetData.get('dataType', 'number')
-
-    _firstCreationDate = ContactMailingStatus.objects.filter(**conditions)
-    _s = {"d": "strftime('%%Y-%%m-%%d', creation_date)"}
-    _firstCreationDate = _firstCreationDate.extra(_s)
-    _firstCreationDate = _firstCreationDate.values('d').distinct().order_by('d')
-    _t = datetime.strptime(_firstCreationDate[0]['d'], '%Y-%m-%d')
-    dates = getDateRange(period, defaultStart=_t)
+    mode = reqestGetData.get('mode', 'accumulate')
+    allDataDates = getAllDataDate()
+    
+#     period = reqestGetData.get('period', 'week')
+# 
+#     _firstCreationDate = ContactMailingStatus.objects.filter(**conditions)
+#     _s = {"d": "strftime('%%Y-%%m-%%d', creation_date)"}
+#     _firstCreationDate = _firstCreationDate.extra(_s)
+#     _firstCreationDate = _firstCreationDate.values('d').distinct().order_by('d')
+#     _t = datetime.strptime(_firstCreationDate[0]['d'], '%Y-%m-%d')
+#     dates = getDateRange(period, defaultStart=_t)
+    
 
     def getContactStatus(conditions):
         contactStatus = ContactMailingStatus.objects.filter(**conditions)
@@ -391,9 +442,16 @@ def getStatisticsTotalData(reqestGetData, subscriberCount, testSubscriberCount,
         return contactStatus
 
     def sortByDatetime(contactStatus):
-        completed = {}
-        for r in contactStatus:
-            date = r['d']
+        completed = OrderedDict()
+        
+        dates = []
+        for c in contactStatus:
+            date = c['d']            
+            if not date in dates:
+                dates.append(date)
+        dates = sorted(dates)
+        
+        for date in dates:
             year = date.split('-')[0]
             month = date.split('-')[1]
             day = date.split('-')[2]
@@ -403,8 +461,10 @@ def getStatisticsTotalData(reqestGetData, subscriberCount, testSubscriberCount,
             completed[date] = queryset
         return completed
 
+    # -- get all date content -------------------------------------------------
     conditions.update(
-        {'creation_date__range': [dates.startDate, dates.endDate]})
+        {'creation_date__range': [allDataDates.startDate, 
+                                  allDataDates.endDate]})
     
     if 'newsletter__id__in' in conditions:
         newsletterIds = conditions.pop('newsletter__id__in')
@@ -413,7 +473,7 @@ def getStatisticsTotalData(reqestGetData, subscriberCount, testSubscriberCount,
         for _id in newsletterIds:
             conditions.update({'newsletter__id': _id})
             _cs = getContactStatus(conditions)
-            _statusCount = countStatus(_cs)
+            _statusCount = countStatus(_cs)[0]
 
         # Merge.
         totalData = []
@@ -431,20 +491,62 @@ def getStatisticsTotalData(reqestGetData, subscriberCount, testSubscriberCount,
         contactStatus = getContactStatus(conditions)
         completed = sortByDatetime(contactStatus)
         totalData = []
+        openIds = []
+        openOnSiteIds = []
+        linkIds = []
         for c in completed:
-            data = countStatus(completed[c])
+            _d = countStatus(completed[c])           
+            open = _d[1]['openId']
+            openOnSite = _d[1]['openOnSiteId']
+            link = _d[1]['clickId']
+            
+            openCount = 0
+            openOnSiteCount = 0
+            linkCount = 0
+            
+            for id in open:
+                if not id in openIds:
+                    if not id in openOnSiteIds:
+                        if not id in linkIds:
+                            openIds.append(id)
+                            openCount += 1
+                            
+            for id in openOnSite:
+                if not id in openOnSiteIds:
+                    openOnSiteIds.append(id)
+                    openOnSiteCount += 1
+                    
+            for id in link:
+                if not id in linkIds:
+                    linkIds.append(id)
+                    linkCount += 1
+                    
+            data = _d[0]
+            data['open'] = openCount
+            data['openOnSite'] = openOnSiteCount
+            data['click'] = linkCount
             totalData.append([c, data])
 
-    allDates = dict(totalData).keys()
-    for day in dates._days:
+    allDates = sorted(dict(totalData).keys())
+    for day in allDataDates._days:        
         if day not in allDates:
-            totalData.append([day, getEmptyDataStructure()])
+            OrderedDict(totalData.append([day, getEmptyDataStructure()]))  
 
-    content = {}
+    allContent = {}
     if dataType == 'number':
-        #content[day] = [(status, value), ...]
-        for (day, data) in totalData:
-            content[day] = [(i, data[i]) for i in data]
+        #allContent[day] = [(status, value), ...]
+        if mode == 'single':
+            for (day, data) in totalData:
+                allContent[day] = [(i, data[i]) for i in data]
+        else:
+            accumulativeData = getEmptyDataStructure()
+            for (day, data) in totalData:
+                result = []
+                for status in data:
+                    accumulativeData[status] += data[status]
+                    result.append((status, accumulativeData[status]))
+                allContent[day] = result
+        
     else:
         def _do(info, subscriberCount, testSubscriberCount):
             rates = []
@@ -464,25 +566,83 @@ def getStatisticsTotalData(reqestGetData, subscriberCount, testSubscriberCount,
                     value = percentage(info[i], totalValue, showPercent=False)
                 
                 rates.append((i, value))
-#                 if i == 'unsubscribe':
-#                     totalValue = subscriberCount
-#                 else:
-#                     totalValue = info['deliverie']
-#                 value = percentage(info[i], totalValue, showPercent=False)
-#                 rates.append((i, value)) # (status, value)
             return rates
+        
+        if mode == 'single':
+            for (day, data) in totalData:
+                result = _do(data, subscriberCount, testSubscriberCount)
+                allContent[day] = result # {'day': (status, value)}
+        else:
+            accumulativeData = getEmptyDataStructure()
 
-        for (day, data) in totalData:
-            result = _do(data, subscriberCount, testSubscriberCount)
-            content[day] = result # {'day': (status, value)}
-
+            for (day, data) in totalData:
+                result = _do(data, subscriberCount, testSubscriberCount)
+                accResult = []
+                for r in result:
+                    status = r[0]
+                    value = r[1]
+                    accumulativeData[status] += value
+                    _result = (status, accumulativeData[status])
+                    accResult.append(_result)
+                allContent[day] = accResult
+    # -- get all date content end ----------------------------------------------
+    
+    # -- get range date content start ------------------------------------------
+    deltaDays = None
+    startDate = reqestGetData.get('startDate', None)    
+    startDate = startDate.encode('ascii','ignore')
+    
+    endDate = reqestGetData.get('endDate', None)
+    endDate = endDate.encode('ascii','ignore')
+    
+    period = reqestGetData.get('period', 'week')    
+    if period == 'week':
+        deltaDays = 7
+        endDate = datetime.now()        
+        startDate = endDate - timedelta(days=deltaDays)
+    elif period == 'month':
+        deltaDays = int(1 * (365 / 12))
+        endDate = datetime.now()
+        startDate = endDate - timedelta(days=deltaDays)
+    elif period == '3month':
+        deltaDays = int(3 * (365 / 12))
+        endDate = datetime.now()
+        startDate = endDate - timedelta(days=deltaDays)
+    dateRange = getDateRange(startDate, endDate)
+    
+    content = {}
+    _days = dateRange._days
+    lastDate = allDates[-1]
+    lastDateObj = datetime.strptime(allDates[-1], '%Y-%m-%d')
+    for _d in _days:
+        if _d in allContent.keys():
+            content[_d] = allContent[_d]
+        else:
+            _emptyData = getEmptyDataStructure()            
+            if mode == 'single':    
+                if dataType == 'number':
+                    content[_d] = [(i, _emptyData[i]) for i in _emptyData]
+                else:
+                    content[_d] = [(i, 0.0) for i in _emptyData]
+            else:
+                _dDateObj = datetime.strptime(_d, '%Y-%m-%d')
+                if _dDateObj > lastDateObj:
+                    content[_d] = allContent[lastDate]
+                else:
+                    if dataType == 'number':
+                        content[_d] = [(i, _emptyData[i]) for i in _emptyData]
+                    else:
+                        content[_d] = [(i, 0.0) for i in _emptyData]              
+   # -- get range date content end --------------------------------------------
+    
+    
     context = {}
     for day in content:
         for (statusName, value) in content[day]:
             dData = context.get(statusName, [])
             dData.append((day, value))
             context[statusName] = dData
-
+                
     # Find the max value of Y-axis.
     tmp = []
     for k in context:
@@ -497,8 +657,8 @@ def getStatisticsTotalData(reqestGetData, subscriberCount, testSubscriberCount,
             'email': context,
         },
         'rateType': dataType,
-        'dateRange': dates._days,
-        'xaxis': {'minDate': dates._days[0], 'maxDate': dates._days[-1]},
+        'dateRange': dateRange._days,
+        'xaxis': {'minDate': dateRange._days[0], 'maxDate': dateRange._days[-1]},
         'yaxis': {'max': yaxisMaxValue},
         'status': _STATUS,
         'settings': {
@@ -536,7 +696,7 @@ def getStatisticsTotalRateData(reqestGetData, subscriberCount,
             conditions.update({'newsletter__id': _id})
             _cs = getContactStatus(conditions)
             
-            _data = countStatus(_cs)
+            _data = countStatus(_cs)[0]
             for _d in _data:
                 v = _data[_d]
                 if _d not in data:
@@ -544,7 +704,7 @@ def getStatisticsTotalRateData(reqestGetData, subscriberCount,
                 data[_d] += v
     else:
         contactStatus = getContactStatus(conditions)
-        data = countStatus(contactStatus)
+        data = countStatus(contactStatus)[0]
 
     rates = []
     
